#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 fragPosLightSpace;
	mat3 TBN;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 brightness;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;

uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;
uniform sampler2D normalMap;
uniform sampler2D shadowMap;
uniform samplerCube cubemapSampler;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL[10];

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL[10];

uniform int dirLightNumber;
uniform int pointLightNumber;

uniform bool normalsActive;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

float calcShadow(vec4 fragPosLightSpace, int i){

	vec3 normal = normalize(vert.normal_world);
	vec3 light = normalize(dirL[i].direction);
	float bias = max(0.002 * (1.0 - dot(normal, dirL[i].direction)), 0.002);

	vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
   	projCoords = projCoords * 0.5 + 0.5;

    	float closestDepth = texture(shadowMap, projCoords.xy).r; 
    	float currentDepth = projCoords.z;

 	float shadow = 0.0;
    	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
    	for(int x = -1; x <= 1; ++x)
    	{
        	for(int y = -1; y <= 1; ++y)
        	{
            		float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            		shadow += currentDepth  -bias> pcfDepth  ? 1.0 : 0.0;        
        	}    
    	}
    	shadow /= 9.0;
    
    	if(projCoords.z > 1.0)
        	shadow = 0.0;
        
    	return shadow;
}

void main() {	
	vec3 n;
	if(normalsActive){
	n = texture(normalMap, vert.uv).rgb;
	n = n * 2.0 - 1.0;
	n = normalize(vert.TBN * n);
	}else{
	n = normalize(vert.normal_world);
	}

	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	vec3 specColor = texture(specularTexture, vert.uv).rgb;
	vec3 cubeColor = texture(cubemapSampler,reflect(normalize(vert.position_world-camera_world),normalize(n))).rgb;

	color = vec4(((cubeColor*specColor + texColor) * materialCoefficients.x)/2, 1); // ambient with cube map
	//color = vec4((specColor + texColor * materialCoefficients.x)/2, 1); // ambient with cube map

	
	// add directional light contribution
	for(int i = 0; i < dirLightNumber; i++){
		//color.rgb = vec3(0,1,0);

		float shadow = calcShadow(vert.fragPosLightSpace, i);
		color.rgb += (1.0 - shadow) * phong(n, -dirL[i].direction, v, dirL[i].color * texColor, materialCoefficients.y, dirL[i].color  * specColor, materialCoefficients.z, specularAlpha, false, vec3(0));
	}	
	
	
	//brightness
	vec3 col = color.rgb;
	vec3 threshold = vec3(0.3, 0.7, 0.07);
	float bright = dot( col, threshold );
    	if(bright > 1.0 ){
        	brightness= vec4(col,1.0);
	}
    	else{
        	brightness= vec4(0.0, 0.0, 0.0, 1.0);
	}
}

